
import redis, re, logging

class Store(object):
    """Simple object storage for Python in Redis
    """

    first_cap_re = re.compile('(.)([A-Z][a-z]+)')
    all_cap_re = re.compile('([a-z0-9])([A-Z])')

    def _class_name_to_assoc_name(self, class_name):
        s1 = self.first_cap_re.sub(r'\1_\2', class_name)
        return self.all_cap_re.sub(r'\1_\2', s1).lower()

    def __init__(self, **kwargs):
        self._redis = redis.StrictRedis(host='localhost', port=6379, db=6)
        self._identity_map = {}
        self._model_classes = kwargs.get('model_classes', [])

        self._classes = {}
        self._assoc_names = {}
        for class_ in self._model_classes:
            assoc_name = self._class_name_to_assoc_name(class_.__name__)
            self._classes[class_] = assoc_name
            self._assoc_names[assoc_name] = class_

    def save(self, object):
        if not hasattr(object, 'id') or not getattr(object, 'id'):
            object.id = self._next_class_id(object.__class__)

        key = self._object_key(object)
        if not key in self._identity_map:
            self._identity_map[key] = object

        self._redis.hmset(key, self._serialize(object))

    def get_by_class_and_id(self, class_, id, reload=False):

        logging.debug(__name__ + ' ' + class_.__name__ + ' ' + str(id))

        key = self._object_key_by_class_and_id(class_, id)

        instance = None
        in_identity_map = key in self._identity_map

        if in_identity_map:
            instance = self._identity_map[key]

        if instance and not reload:
            return instance

        raw_attrs = self._redis.hgetall(key)
        if not raw_attrs:
            if in_identity_map:
                del self._identity_map[key]
            return None

        if not in_identity_map:
            instance = class_()

        clean_attrs = {}
        for attr_name in raw_attrs:
            id_attr_match = re.search(r'(.+)_id', attr_name)
            if id_attr_match and id_attr_match.group(1) in self._assoc_names:
                real_attr_name = id_attr_match.group(1)
                assoc_class_ = self._assoc_names[real_attr_name]
                clean_attrs[real_attr_name] = self.get_by_class_and_id(assoc_class_, raw_attrs[attr_name])
            clean_attrs[attr_name] = raw_attrs[attr_name]

        instance.__dict__.update(clean_attrs)
        return instance

    def delete(self, object):
        pass

    def reload(self, object):
        key = self._object_key(object)
        if key in self._identity_map and self._identity_map[key] != object:
            raise Exception('Trying to reload object which was not loaded from this Store')
        self.get_by_class_and_id(object.__class__, object.id, reload=True)

    def map(self, map_name, key, object):
        pass

    def get_mapped(self, map_name, key):
        return []

    def _next_class_id(self, class_):
        return str(self._redis.incr(self._class_id_counter_name(class_)))

    def _class_id_counter_name(self, class_):
        return class_.__name__ + '_id'

    def _object_key_by_class_and_id(self, class_, id):
        return class_.__name__ + ':' + str(id)

    def _object_key(self, object):
        return self._object_key_by_class_and_id(object.__class__, object.id)

    def _serialize(self, object):
        attributes = {}
        for attr_name in object.__dict__:

            if attr_name in self._assoc_names:
                if not object.__dict__[attr_name].id:
                    # TODO Beware infinite recursion!
                    self.save(object.__dict__[attr_name])
                attributes[attr_name + '_id'] = object.__dict__[attr_name].id
            elif re.search(r'(.+)_id', attr_name):
                attributes[attr_name] = object.__dict__[attr_name]
            else:
                attributes[attr_name] = object.__dict__[attr_name]

        return attributes

    def _empty(self):
        self._redis.flushdb()
        self._identity_map = {}

    def get_all(self, class_):
        keys = self._redis.keys(class_.__name__ + ':*')
        all = []
        for key in keys:
            id_match = re.search(r':([^:]+)$', key)
            if id_match:
                obj = self.get_by_class_and_id(class_, id_match.group(1))
                if obj:
                    all.append(obj)
        return all
