
from sobstore.test import *

import dummies

class SobstoreTest(unittest.TestCase):

    def setUp(self):
        self.store = Store(model_classes=[dummies.Customer,
                                          dummies.Product,
                                          dummies.ProductCategory,
                                          dummies.Order])
        self.store._empty()
        self.c1 = dummies.Customer(name='Customer 1')
        self.c2 = dummies.Customer(name='Customer 2')
        self.books_category = dummies.ProductCategory(name='Books')
        self.food_category = dummies.ProductCategory(name='Food')

    def test_get_all_empty(self):
        self.store.save(self.c1)
        customer_id = self.c1.id
        self.assertEqual(1, len(self.store.get_all(dummies.Customer)))
        self.store._empty()
        self.assertEqual(0, len(self.store.get_all(dummies.Customer)))
        self.assertIsNone(self.store.get_by_class_and_id(dummies.Customer, customer_id))

    def test_save_assigns_unique_id(self):
        self.assertIsNone(self.c1.id)
        self.store.save(self.c1)
        self.assertIsNotNone(self.c1.id)

        self.assertIsNone(self.c2.id)
        self.store.save(self.c2)
        self.assertIsNotNone(self.c2.id)
        self.assertNotEqual(self.c1.id, self.c2.id)

    def test_reload(self):
        self.store.save(self.c1)
        self.store.reload(self.c1)
        self.assertEqual('Customer 1', self.c1.name)

    def test_get_by_class_and_id(self):
        self.store.save(self.c1)

        c1_loaded = self.store.get_by_class_and_id(dummies.Customer, self.c1.id)
        self.assertEqual(self.c1, c1_loaded)

        self.store.save(self.c2)
        c2_loaded = self.store.get_by_class_and_id(dummies.Customer, self.c2.id)
        self.assertNotEqual(c1_loaded, c2_loaded)

    def test_serialize(self):
        self.store.save(self.books_category)

        book1 = dummies.Product(name='PHP for dummies', product_category=self.books_category)
        s1 = self.store._serialize(book1)
        self.assertIsInstance(s1, dict)
        self.assertIn('name', s1)
        self.assertIn('product_category_id', s1)
        self.assertEqual('PHP for dummies', s1.get('name'))
        self.assertEqual(self.books_category.id, s1.get('product_category_id'))

        book2 = dummies.Product(name='Python for programmers', product_category_id=self.books_category.id)
        s2 = self.store._serialize(book2)
        self.assertEqual('Python for programmers', s2.get('name'))
        self.assertEqual(self.books_category.id, s2.get('product_category_id'))

    def test_save_associations(self):
        self.store.save(self.books_category)

        book1 = dummies.Product(name='PHP for dummies', product_category=self.books_category)
        self.store.save(book1)
        self.store.reload(book1)

        book2 = dummies.Product(name='Python for programmers', product_category_id=self.books_category.id)
        self.store.save(book2)
        self.store.reload(book2)

        self.assertEqual(book1.product_category_id, book2.product_category_id)
        self.assertEqual(book1.product_category, book2.product_category)

    def test_get_by_class_and_id_with_reload(self):
        potato = dummies.Product(name='Potato', product_category=self.food_category)
        self.store.save(potato)

        potato_by_id = self.store.get_by_class_and_id(dummies.Product, potato.id)

        potato_reloaded = self.store.get_by_class_and_id(dummies.Product, potato.id, reload=True)

        self.assertIsInstance(potato_by_id, dummies.Product)
        self.assertEquals(potato_by_id, potato)
        self.assertEquals(potato_by_id, potato_reloaded)

    def test_complete_story(self):

        store = Store(model_classes=[dummies.Product, dummies.ProductCategory])
        store._empty()

        food = dummies.ProductCategory(name='Food')
        potato = dummies.Product(name='Potato', product_category=food)
        tomato = dummies.Product(name='Tomato', product_category=food)

        books = dummies.ProductCategory(name='Books')
        python_book = dummies.Product(name='Dive into Python', product_category=books)
        php_book = dummies.Product(name='PHP in 24 Hours', product_category=books)

        for obj in [food, potato, tomato, books, python_book, php_book]:
            store.save(obj)

        products = store.get_all(dummies.Product)
        self.assertIsInstance(products, list)
        self.assertEquals(4, len(products))
        self.assertIn(potato, products)
        self.assertIn(tomato, products)
        self.assertIn(python_book, products)
        self.assertIn(php_book, products)
        self.assertNotIn(food, products)
        self.assertNotIn(books, products)

        categories = store.get_all(dummies.ProductCategory)
        self.assertEquals(2, len(categories))
        self.assertIn(food, categories)
        self.assertIn(books, categories)
